from datetime import datetime
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from jumbo.core.urlresolvers import reverse
from jumbo.middleware.thread_local import get_request
from openvpn.models import *

class UtentiList(object):
    list_display = ['username', 'nome', 'cognome', 'get_edit_icon']
    sort_field = {
        'username'        : 'username',
    }
    default_order_by = 'username'
    model = Utenti

    def check_permissions(self, request):
        if request.user.has_perm('openvpn.can_view_utenti'):
            return True
        return False

    def get_edit_icon(self, result):
        return "<a href=%(reverse_url)s><img src='%(url)sjumbo/img/edit.gif' alt='%(window_title)s' /></a>" % {
            'reverse_url':reverse('openvpn-utenti-edit', result.id),
            'url':settings.MEDIA_URL,
            'window_title':_("Edit User")
        }
    get_edit_icon.short_description = _("E")

    def get_del_icon(self, result):
        return """<a href='%(reverse_url)s' onclick=\"jshowAddAnotherPopup(this, '%(window_title)s', 500, 500, 200, 200);return false;\">
                    <img src='%(url)sjumbo/img/del.png' title=\"%(window_title)s\" /></a>""" % {
                                        'reverse_url':reverse('openvpn-utenti-delete', result.id), 
                                        'url':settings.MEDIA_URL, 
                                        'window_title':_("Delete User")}
    get_del_icon.short_description = _("Del")

class CertificateList(object):
    list_display = ['status', 'user', 'description', 'start_date', 'stop_date', 
                    'cn', 'group', 'get_edit_icon', 'get_download_icon', 'get_del_icon']
    sort_field = {
        'user'        : 'user',
    }
    default_order_by = 'user'
    model = Certificate

    def check_permissions(self, request):
        if request.user.has_perm('openvpn.can_view_certificate'):
            return True
        return False

    def get_edit_icon(self, result):
        return "<a href=%(reverse_url)s><img src='%(url)sjumbo/img/edit.gif' alt='%(window_title)s' /></a>" % {
            'reverse_url':reverse('openvpn-certificate-edit', result.id),
            'url':settings.MEDIA_URL,
            'window_title':_("Edit Certificate")
        }
    get_edit_icon.short_description = _("E")

    def get_download_icon(self, result):
        return "<a href=%(reverse_url)s><img src='%(url)sjumbo/img/download.gif' alt='%(window_title)s' /></a>" % {
            'reverse_url':reverse('openvpn-certificate-download', result.id),
            'url':settings.MEDIA_URL,
            'window_title':_("Download Certificate")
        }
    get_download_icon.short_description = _("D")

    def get_del_icon(self, result):
        return """<a href='%(reverse_url)s' onclick=\"jshowAddAnotherPopup(this, '%(window_title)s', 500, 500, 200, 200);return false;\">
                    <img src='%(url)sjumbo/img/del.png' title=\"%(window_title)s\" /></a>""" % {
                                        'reverse_url':reverse('openvpn-certificate-delete', result.id), 
                                        'url':settings.MEDIA_URL, 
                                        'window_title':_("Delete Certificate")}
    get_del_icon.short_description = _("Del")

class FwruleList(object):
    list_display = ['get_group', 'table', 'chain', 'ip_dest', 'protocol', 'port_dest', 'action', 'order', 
                    'get_edit_icon', 'get_del_icon']
    default_order_by = 'group'
    model = FwRule

    def check_permissions(self, request):
        if request.user.has_perm('openvpn.can_view_fwrule'):
            return True
        return False

    def get_group(self, result):
        return ','.join([g.name for g in result.group.all() ])
    get_group.short_description = _("Groups")

    def get_edit_icon(self, result):
        return "<a href=%(reverse_url)s><img src='%(url)sjumbo/img/edit.gif' alt='%(window_title)s' /></a>" % {
            'reverse_url':reverse('openvpn-fwrule-edit', result.id),
            'url':settings.MEDIA_URL,
            'window_title':_("Edit Rule")
        }
    get_edit_icon.short_description = _("E")

    def get_del_icon(self, result):
        return """<a href='%(reverse_url)s' onclick=\"jshowAddAnotherPopup(this, '%(window_title)s', 500, 500, 200, 200);return false;\">
                    <img src='%(url)sjumbo/img/del.png' title=\"%(window_title)s\" /></a>""" % {
                                        'reverse_url':reverse('openvpn-fwrule-delete', result.id), 
                                        'url':settings.MEDIA_URL, 
                                        'window_title':_("Delete Rule")}
    get_del_icon.short_description = _("Del")

class FwgroupList(object):
    list_display = ['name', 'get_edit_icon', 'get_del_icon']
    default_order_by = 'name'
    model = FwGroup

    def check_permissions(self, request):
        if request.user.has_perm('openvpn.can_view_fwgroup'):
            return True
        return False

    def get_edit_icon(self, result):
        return "<a href=%(reverse_url)s><img src='%(url)sjumbo/img/edit.gif' alt='%(window_title)s' /></a>" % {
            'reverse_url':reverse('openvpn-fwgroup-edit', result.id),
            'url':settings.MEDIA_URL,
            'window_title':_("Edit Group")
        }
    get_edit_icon.short_description = _("E")

    def get_del_icon(self, result):
        return """<a href='%(reverse_url)s' onclick=\"jshowAddAnotherPopup(this, '%(window_title)s', 500, 500, 200, 200);return false;\">
                    <img src='%(url)sjumbo/img/del.png' title=\"%(window_title)s\" /></a>""" % {
                                        'reverse_url':reverse('openvpn-fwgroup-delete', result.id), 
                                        'url':settings.MEDIA_URL, 
                                        'window_title':_("Delete Group")}
    get_del_icon.short_description = _("Del")

class ConnectionList(object):
    list_display = ['certificate', 'start_date', 'stop_date', 'ip_client']
    sort_field = {
        'stop_date'        : 'stop_date',
    }
    default_order_by = '-stop_date'
    model = Connection

    def check_permissions(self, request):
        if request.user.has_perm('openvpn.can_view_connection'):
            return True
        return False
