import sys
from django.core.management.base import BaseCommand, CommandError
from optparse import make_option

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('-c', '--common_name', dest='common-name',
            help="The Certificate's common name."),
        make_option('-i', '--ip_client', dest='ip-client',
            help='Specifies the IP Client.'),
    )
    help = "Save an open connection for the user."

    # Validation is called explicitly each time the server is reloaded.
    requires_model_validation = False

    def handle(self, *args, **options):
        from openvpn.models import Certificate, Connection

        cn = options.get('common-name',None)
        ip_client = options.get('ip-client', '0.0.0.0')

        if cn:
            c = Certificate.objects.get(cn=cn)
            if not c.status:
                sys.exit(1)
            conn = Connection(certificate_id=c.id)
            conn.ip_client = ip_client
            conn.save()
        else:
            sys.exit(1)
